@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 module YARD
   module CodeObjects
     # A list of code objects. This array acts like a set (no unique items)
@@ -24,9 +25,10 @@ module YARD
         end
         self
       end
-      alias_method :<<, :push
+      alias << push
     end
 
+    extend NamespaceMapper
 
     # Namespace separator
     NSEP = '::'
@@ -49,35 +51,47 @@ module YARD
     # Regular expression to match constant name
     CONSTANTMATCH = /[A-Z]\w*/
 
+    # Regular expression to match the beginning of a constant
+    CONSTANTSTART = /^[A-Z]/
+
     # Regular expression to match namespaces (const A or complex path A::B)
-    NAMESPACEMATCH = /(?:(?:#{NSEPQ})?#{CONSTANTMATCH})+/
+    NAMESPACEMATCH = /(?:(?:#{NSEPQ}\s*)?#{CONSTANTMATCH})+/
 
     # Regular expression to match a method name
-    METHODNAMEMATCH = /[a-zA-Z_]\w*[!?=]?|[-+~]\@|<<|>>|=~|===?|<=>|[<>]=?|\*\*|[-\/+%^&*~`|]|\[\]=?/
+    METHODNAMEMATCH = %r{[a-zA-Z_]\w*[!?=]?|[-+~]\@|<<|>>|=~|===?|![=~]?|<=>|[<>]=?|\*\*|[-/+%^&*~`|]|\[\]=?}
 
     # Regular expression to match a fully qualified method def (self.foo, Class.foo).
     METHODMATCH = /(?:(?:#{NAMESPACEMATCH}|[a-z]\w*)\s*(?:#{CSEPQ}|#{NSEPQ})\s*)?#{METHODNAMEMATCH}/
 
     # All builtin Ruby exception classes for inheritance tree.
-    BUILTIN_EXCEPTIONS = ["SecurityError", "Exception", "NoMethodError", "FloatDomainError",
-      "IOError", "TypeError", "NotImplementedError", "SystemExit", "Interrupt", "SyntaxError",
-      "RangeError", "NoMemoryError", "ArgumentError", "ThreadError", "EOFError", "RuntimeError",
-      "ZeroDivisionError", "StandardError", "LoadError", "NameError", "LocalJumpError", "SystemCallError",
-      "SignalException", "ScriptError", "SystemStackError", "RegexpError", "IndexError"]
+    BUILTIN_EXCEPTIONS = ["ArgumentError", "ClosedQueueError", "EncodingError",
+      "EOFError", "Exception", "FiberError", "FloatDomainError", "IndexError",
+      "Interrupt", "IOError", "KeyError", "LoadError", "LocalJumpError",
+      "NameError", "NoMemoryError", "NoMethodError", "NotImplementedError",
+      "RangeError", "RegexpError", "RuntimeError", "ScriptError", "SecurityError",
+      "SignalException", "StandardError", "StopIteration", "SyntaxError",
+      "SystemCallError", "SystemExit", "SystemStackError", "ThreadError",
+      "TypeError", "UncaughtThrowError", "ZeroDivisionError"]
+
     # All builtin Ruby classes for inheritance tree.
     # @note MatchingData is a 1.8.x legacy class
-    BUILTIN_CLASSES = ["TrueClass", "Array", "Dir", "Struct", "UnboundMethod", "Object", "Fixnum", "Float",
-      "ThreadGroup", "MatchingData", "MatchData", "Proc", "Binding", "Class", "Time", "Bignum", "NilClass", "Symbol",
-      "Numeric", "String", "Data", "MatchData", "Regexp", "Integer", "File", "IO", "Range", "FalseClass",
-      "Method", "Continuation", "Thread", "Hash", "Module"] + BUILTIN_EXCEPTIONS
+    BUILTIN_CLASSES = ["Array", "Bignum", "Binding", "Class", "Complex",
+      "ConditionVariable", "Data", "Dir", "Encoding", "Enumerator", "FalseClass",
+      "Fiber", "File", "Fixnum", "Float", "Hash", "IO", "Integer", "MatchData",
+      "Method", "Module", "NilClass", "Numeric", "Object", "Proc", "Queue",
+      "Random", "Range", "Rational", "Regexp", "RubyVM", "SizedQueue", "String",
+      "Struct", "Symbol", "Thread", "ThreadGroup", "Time", "TracePoint",
+      "TrueClass", "UnboundMethod"] + BUILTIN_EXCEPTIONS
+
     # All builtin Ruby modules for mixin handling.
-    BUILTIN_MODULES = ["ObjectSpace", "Signal", "Marshal", "Kernel", "Process", "GC", "FileTest", "Enumerable",
-      "Comparable", "Errno", "Precision", "Math"]
+    BUILTIN_MODULES = ["Comparable", "Enumerable", "Errno", "FileTest", "GC",
+      "Kernel", "Marshal", "Math", "ObjectSpace", "Precision", "Process", "Signal"]
+
     # All builtin Ruby classes and modules.
     BUILTIN_ALL = BUILTIN_CLASSES + BUILTIN_MODULES
 
     # Hash of {BUILTIN_EXCEPTIONS} as keys and true as value (for O(1) lookups)
-    BUILTIN_EXCEPTIONS_HASH = BUILTIN_EXCEPTIONS.inject({}) {|h,n| h.update(n => true) }
+    BUILTIN_EXCEPTIONS_HASH = BUILTIN_EXCEPTIONS.inject({}) {|h, n| h.update(n => true) }
 
     # +Base+ is the superclass of all code objects recognized by YARD. A code
     # object is any entity in the Ruby language (class, method, module). A
@@ -102,15 +116,23 @@ module YARD
     # though a DSL might create a custom {NamespaceObject} to describe a
     # specific set of objects.
     #
+    # == Separators
+    # Custom classes with different separator tokens should define their own
+    # separators using the {NamespaceMapper.register_separator} method. The
+    # standard Ruby separators have already been defined ('::', '#', '.', etc).
+    #
     # @abstract This class should not be used directly. Instead, create a
-    #   subclass that implements {#path}, {#sep} or {#type}.
+    #   subclass that implements {#path}, {#sep} or {#type}. You might also
+    #   need to register custom separators if {#sep} uses alternate separator
+    #   tokens.
     # @see Registry
     # @see #path
     # @see #[]=
     # @see NamespaceObject
+    # @see NamespaceMapper.register_separator
     class Base
       # The files the object was defined in. To add a file, use {#add_file}.
-      # @return [Array<String>] a list of files
+      # @return [Array<Array(String, Integer)>] a list of files
       # @see #add_file
       attr_reader :files
 
@@ -136,9 +158,12 @@ module YARD
       # @return [String] a line of source
       attr_accessor :signature
 
-      # The documentation string associated with the object
+      # The non-localized documentation string associated with the object
       # @return [Docstring] the documentation string
-      attr_reader :docstring
+      # @since 0.8.4
+      attr_reader :base_docstring
+      undef base_docstring
+      def base_docstring; @docstring end
 
       # Marks whether or not the method is conditionally defined at runtime
       # @return [Boolean] true if the method is conditionally defined at runtime
@@ -167,10 +192,12 @@ module YARD
             namespace = Proxy.new(namespace.namespace, namespace.value)
           end
 
-          if name.to_s[0,2] == NSEP
+          if name.to_s[0, 2] == NSEP
             name = name.to_s[2..-1]
             namespace = Registry.root
-          elsif name =~ /(?:#{NSEPQ})([^:]+)$/
+          end
+
+          if name =~ /(?:#{NSEPQ})([^:]+)$/
             return new(Proxy.new(namespace, $`), $1, *args, &block)
           end
 
@@ -204,9 +231,9 @@ module YARD
       # @yield [self] a block to perform any extra initialization on the object
       # @yieldparam [Base] self the newly initialized code object
       # @return [Base] the newly created object
-      def initialize(namespace, name, *args, &block)
+      def initialize(namespace, name, *)
         if namespace && namespace != :root &&
-            !namespace.is_a?(NamespaceObject) && !namespace.is_a?(Proxy)
+           !namespace.is_a?(NamespaceObject) && !namespace.is_a?(Proxy)
           raise ArgumentError, "Invalid namespace object: #{namespace}"
         end
 
@@ -216,9 +243,8 @@ module YARD
         @source_type = :ruby
         @visibility = :public
         @tags = []
-        @docstring = Docstring.new('', self)
-        @docstring_extra = nil
-        @docstring_extra_tags = nil
+        @docstrings = {}
+        @docstring = Docstring.new!('', [], self)
         @namespace = nil
         self.namespace = namespace
         yield(self) if block_given?
@@ -235,7 +261,7 @@ module YARD
           ivar = "@#{ivar}"
           other.instance_variable_set(ivar, instance_variable_get(ivar))
         end
-        other.docstring = docstring.to_raw
+        other.docstring = @docstring.to_raw
         other
       end
 
@@ -246,7 +272,7 @@ module YARD
       # @return [String] if prefix is true, prefix + the name as a String.
       #   This must be implemented by the subclass.
       def name(prefix = false)
-        prefix ? @name.to_s : @name
+        prefix ? @name.to_s : (defined?(@name) && @name)
       end
 
       # Associates a file with a code object, optionally adding the line where it was defined.
@@ -363,19 +389,25 @@ module YARD
         end
       end
 
-      undef docstring
-      def docstring
-        return @docstring if !@docstring_extra
-        case @docstring
-        when Proxy
-          return @docstring_extra
-        when Base
-          @docstring = @docstring.docstring + @docstring_extra
-          @docstring.add_tag(*@docstring_extra_tags)
-          @docstring_extra = nil
-          @docstring_extra_tags = nil
+      # The documentation string associated with the object
+      #
+      # @param [String, I18n::Locale] locale (I18n::Locale.default)
+      #   the locale of the documentation string.
+      # @return [Docstring] the documentation string
+      def docstring(locale = I18n::Locale.default)
+        if locale.nil?
+          @docstring.resolve_reference
+          return @docstring
         end
-        @docstring
+
+        if locale.is_a?(String)
+          locale_name = locale
+          locale = nil
+        else
+          locale_name = locale.name
+        end
+        @docstrings[locale_name] ||=
+          translate_docstring(locale || Registry.locale(locale_name))
       end
 
       # Attaches a docstring to a code object by parsing the comments attached to the statement
@@ -385,17 +417,9 @@ module YARD
       #   the comments attached to the code object to be parsed
       #   into a docstring and meta tags.
       def docstring=(comments)
-        if comments =~ /\A\s*\(see (\S+)\s*\)(?:\s|$)/
-          path, extra = $1, $'
-          @docstring_extra = Docstring.new(extra, self)
-          @docstring_extra_tags = Docstring === comments ? comments.tags : []
-          @docstring_extra.add_tag(*@docstring_extra_tags)
-          @docstring = Proxy.new(namespace, path)
-        else
-          @docstring_extra = nil
-          @docstring_extra_tags = nil
-          @docstring = Docstring === comments ? comments : Docstring.new(comments, self)
-        end
+        @docstrings.clear
+        @docstring = Docstring === comments ?
+          comments : Docstring.new(comments, self)
       end
 
       # Default type is the lowercase class name without the "Object" suffix.
@@ -403,7 +427,7 @@ module YARD
       #
       # @return [Symbol] the type of code object this represents
       def type
-        self.class.name.split(/#{NSEPQ}/).last.gsub(/Object$/, '').downcase.to_sym
+        self.class.name.split('::').last.gsub(/Object$/, '').downcase.to_sym
       end
 
       # Represents the unique path of the object. The default implementation
@@ -417,12 +441,22 @@ module YARD
       # @see #sep
       def path
         @path ||= if parent && !parent.root?
-          [parent.path, name.to_s].join(sep)
-        else
-          name.to_s
-        end
+                    [parent.path, name.to_s].join(sep)
+                  else
+                    name.to_s
+                  end
+      end
+      alias to_s path
+
+      # @note
+      #   Override this method if your object has a special title that does
+      #   not match the {#path} attribute value. This title will be used
+      #   when linking or displaying the object.
+      # @return [String] the display title for an object
+      # @see 0.8.4
+      def title
+        path
       end
-      alias_method :to_s, :path
 
       # @param [Base, String] other another code object (or object path)
       # @return [String] the shortest relative path from this object to +other+
@@ -438,11 +472,8 @@ module YARD
         common = [path, other].join(" ").match(/^(\S*)\S*(?: \1\S*)*$/)[1]
         common = path unless common =~ /(\.|::|#)$/
         common = common.sub(/(\.|::|#)[^:#\.]*?$/, '') if same_parent
-        if %w(. :).include?(common[-1,1]) || other[common.size,1] == '#'
-          suffix = ''
-        else
-          suffix = '(::|\.)'
-        end
+        suffix = %w(. :).include?(common[-1, 1]) || other[common.size, 1] == '#' ?
+          '' : '(::|\.)'
         result = other.sub(/^#{Regexp.quote common}#{suffix}/, '')
         result.empty? ? other : result
       end
@@ -462,6 +493,7 @@ module YARD
       # @see Templates::Engine#render
       def format(options = {})
         options = options.merge(:object => self)
+        options = options.merge(:type => type) unless options[:type]
         Templates::Engine.render(options)
       end
 
@@ -492,8 +524,8 @@ module YARD
         end
       end
 
-      alias_method :parent, :namespace
-      alias_method :parent=, :namespace=
+      alias parent namespace
+      alias parent= namespace=
 
       # Gets a tag from the {#docstring}
       # @see Docstring#tag
@@ -507,6 +539,14 @@ module YARD
       # @see Docstring#has_tag?
       def has_tag?(name); docstring.has_tag?(name) end
 
+      # Add tags to the {#docstring}
+      # @see Docstring#add_tag
+      # @since 0.8.4
+      def add_tag(*tags)
+        @docstrings.clear
+        @docstring.add_tag(*tags)
+      end
+
       # @return whether or not this object is a RootObject
       def root?; false end
 
@@ -530,7 +570,7 @@ module YARD
       # @since 0.8.0
       def copyable_attributes
         vars = instance_variables.map {|ivar| ivar.to_s[1..-1] }
-        vars -= %w(docstring namespace name path)
+        vars -= %w(docstring docstrings namespace name path)
         vars
       end
 
@@ -541,11 +581,30 @@ module YARD
       # @param [String] source the source code to format
       # @return [String] formatted source
       def format_source(source)
-        source.chomp!
+        source = source.chomp
         last = source.split(/\r?\n/).last
         indent = last ? last[/^([ \t]*)/, 1].length : 0
         source.gsub(/^[ \t]{#{indent}}/, '')
       end
+
+      def translate_docstring(locale)
+        @docstring.resolve_reference
+        return @docstring if locale.nil?
+
+        text = I18n::Text.new(@docstring)
+        localized_text = text.translate(locale)
+        docstring = Docstring.new(localized_text, self)
+        @docstring.tags.each do |tag|
+          if tag.is_a?(Tags::Tag)
+            localized_tag = tag.clone
+            localized_tag.text = I18n::Text.new(tag.text).translate(locale)
+            docstring.add_tag(localized_tag)
+          else
+            docstring.add_tag(tag)
+          end
+        end
+        docstring
+      end
     end
   end
 end
