@@ -20,7 +20,11 @@ class YARD::Handlers::Ruby::ConstantHandler < YARD::Handlers::Ruby::Base
 
   def process_constant(statement)
     name = statement[0].source
-    value = statement[1].source
+    value = if statement[1].type == :dyna_symbol
+              statement[1].first.source.intern.inspect
+            else
+              statement[1].source
+            end
     obj = P(namespace, name)
     if obj.is_a?(NamespaceObject) && obj.namespace == namespace
       raise YARD::Parser::UndocumentableError, "constant for existing #{obj.type} #{obj}"
@@ -35,6 +39,7 @@ class YARD::Handlers::Ruby::ConstantHandler < YARD::Handlers::Ruby::Base
     if lhs.type == :const
       klass = create_class(lhs[0], P(:Struct))
       create_attributes(klass, extract_parameters(statement[1]))
+      parse_block(statement[1].block[1], :namespace => klass) unless statement[1].block.nil?
     else
       raise YARD::Parser::UndocumentableError, "Struct assignment to #{statement[0].source}"
     end
