@@ -1,6 +1,7 @@
+# frozen_string_literal: true
 require File.dirname(__FILE__) + "/spec_helper"
 
-describe YARD::Templates::Engine.template(:default, :constant) do
+RSpec.describe YARD::Templates::Engine.template(:default, :constant) do
   describe "fully dressed constant" do
     it "renders text format correctly" do
       YARD.parse_string <<-'eof'
@@ -37,4 +38,4 @@ describe YARD::Templates::Engine.template(:default, :constant) do
       text_equals(Registry.at('MYCONST').format(text_options), :constant003)
     end
   end
-end
\ No newline at end of file
+end
