@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 def init
   return if object.docstring.blank? && !object.has_tag?(:api)
   sections :index, [:private, :deprecated, :abstract, :todo, :note, :returns_void, :text], T('tags')
@@ -40,10 +41,12 @@ def docstring_text
   unless object.tags(:overload).size == 1 && object.docstring.empty?
     text = object.docstring
   end
-  
+
   if text.strip.empty? && object.tags(:return).size == 1 && object.tag(:return).text
-    text = object.tag(:return).text.gsub(/\A([a-z])/) {|x| x.upcase }
+    text = object.tag(:return).text.gsub(/\A([a-z])/, &:downcase)
+    text = "Returns #{text}" unless text.empty? || text =~ /^\s*return/i
+    text = text.gsub(/\A([a-z])/, &:upcase)
   end
-  
+
   text.strip
-end
\ No newline at end of file
+end
